@@ -13,94 +13,95 @@
 # limitations under the License.
 
 """
-Fixed Storage Capacity
-======================
+.. admonition:: Fixed storage capacity circular array.**
 
-**Circular array with fixed storage capacity.**
-
-- O(1) pops and pushes either end 
-- O(1) indexing, does not support slicing
-- fixed total storage capacity
-- iterable, can safely mutate while iterators continue iterating over previous state
-- comparisons compare identity before equality, like builtins
-- in boolean context, falsy when either empty or full, otherwise truthy
-- factory function ``caf`` produces a fixed storage capacity circular array from its arguments
+    - O(1) pops and pushes either end
+    - O(1) indexing, does not support slicing
+    - fixed total storage capacity
+    - iterable, safely mutates while iterators iterating over previous state
+    - comparisons compare identity before equality, like builtins
+    - in boolean context, falsy when either empty or full, otherwise truthy
+    - function ``caf`` produces fixed capacity circular array from arguments
 
 """
+
 from collections.abc import Callable, Iterable, Iterator
-from typing import cast, Final
+from typing import cast, Final, overload
+from pythonic_fp.gadgets.sentinels.novalue import NoValue
 
 __all__ = ['CAF', 'caf']
 
+nada: Final[NoValue] = NoValue()
 
-class CAF[I]():
 
+class CAF[I]:
     __slots__ = '_items', '_cnt', '_cap', '_front', '_rear'
 
-    def __init__(
-            self,
-            items: Iterable[I] | None = None,
-            capacity: int = 2
-        ) -> None:
+    def __init__(self, *items: Iterable[I], cap: int = 2) -> None:
         """
-        :param items: Optional iterable to initial populate circular array.
-        :param capacity: Fixed storage capacity of circular array.
-        :raises TypeError: When ``items`` not Iterable,
+        :param items: Optionally takes a single iterable to
+                      initially populate the circular array.
+        :param cap: Minimum fixed storage capacity of circular array.
+        :raises TypeError: When ``items[0]`` not iterable,
+        :raises ValueError: If more than 1 iterable is given.
         """
-        capacity = max(2, capacity)
-        if items is None:
-            self._items: list[I | None] = [None]*capacity
-            count = 0
+        cap = max(2, cap)
+        if (size := len(items)) > 1:
+            msg = f'CAF expects at most 1 argument, got {size}'
+            raise ValueError(msg)
+        if size:
+            values: list[I | NoValue] = list(cast(Iterable[I | NoValue], items[0]))
+            cnt = len(values)
+            cap = max(cnt, cap)
+            self._items = values + [nada] * (cap - cnt)
         else:
-            dlist: list[I | None] = list(items)
-            count = len(dlist)
-            capacity = max(count, capacity)
-            self._items = dlist + [None]*(capacity - count)
-        self._cap: Final[int] = capacity
-        self._cnt = count
-        if count == 0:
+            self._items = [nada] * cap
+            cnt = 0
+        self._cap: Final[int] = cap
+        self._cnt = cnt
+        if cnt == 0:
             self._front = 0
-            self._rear = capacity - 1
+            self._rear = cap - 1
         else:
             self._front = 0
-            self._rear = count - 1
+            self._rear = cnt - 1
 
     def __iter__(self) -> Iterator[I]:
         if self._cnt > 0:
             (
-                    capacity,
-                    rear,
-                    position,
-                    current_state,
+                cap,
+                rear,
+                position,
+                current_state,
             ) = (
-                    self._cap,
-                    self._rear,
-                    self._front,
-                    self._items.copy(),
-                )
+                self._cap,
+                self._rear,
+                self._front,
+                self._items.copy(),
+            )
 
             while position != rear:
                 yield cast(I, current_state[position])
-                position = (position + 1) % capacity
+                position = (position + 1) % cap
             yield cast(I, current_state[position])
 
     def __reversed__(self) -> Iterator[I]:
         if self._cnt > 0:
             (
-                    capacity,
-                    front,
-                    position,
-                    current_state,
+                cap,
+                front,
+                position,
+                current_state,
             ) = (
-                    self._cap,
-                    self._front,
-                    self._rear,
-                    self._items.copy(),
-                )
+                self._cap,
+                self._front,
+                self._rear,
+                self._items.copy(),
+            )
 
             while position != front:
                 yield cast(I, current_state[position])
-                position = (position - 1) % capacity
+                position = (position - 1) % cap
             yield cast(I, current_state[position])
 
     def __repr__(self) -> str:
@@ -150,49 +151,61 @@ class CAF[I]():
     def __delitem__(self, idx: int) -> None:
         item_list = list(self)
         del item_list[idx]
-        _ca = CAF(item_list, self._cap)
+        _ca = CAF(item_list, cap = self._cap)
         (
-                self._items,
-                self._cnt,
-                self._front,
-                self._rear,
+            self._items,
+            self._cnt,
+            self._front,
+            self._rear,
         ) = (
-                _ca._items,
-                _ca._cnt,
-                _ca._front,
-                _ca._rear,
-            )
+            _ca._items,
+            _ca._cnt,
+            _ca._front,
+            _ca._rear,
+        )
         del _ca
 
     def __eq__(self, other: object) -> bool:
+        """
+        :param other: The object to be compared to.
+        :returns: ``True`` if object is another CAF whose items compare
+                  as equal to the corresponding items in the CAF,
+                  otherwise ``False``.
+        """
         if self is other:
             return True
         if not isinstance(other, type(self)):
             return False
 
         (
-                front1,
-                count1,
-                capacity1,
-                front2,
-                count2,
-                capacity2,
+            front1,
+            cnt1,
+            cap1,
+            front2,
+            cnt2,
+            cap2,
         ) = (
-                self._front,
-                self._cnt,
-                self._cap,
-                other._front,
-                other._cnt,
-                other._cap,
-            )
+            self._front,
+            self._cnt,
+            self._cap,
+            other._front,
+            other._cnt,
+            other._cap,
+        )
 
-        if count1 != count2:
+        if cnt1 != cnt2:
             return False
 
-        for nn in range(count1):
-            if self._items[(front1 + nn) % capacity1] is other._items[(front2 + nn) % capacity2]:
+        for nn in range(cnt1):
+            if (
+                self._items[(front1 + nn) % cap1]
+                is other._items[(front2 + nn) % cap2]
+            ):
                 continue
-            if self._items[(front1 + nn) % capacity1] != other._items[(front2 + nn) % capacity2]:
+            if (
+                self._items[(front1 + nn) % cap1]
+                != other._items[(front2 + nn) % cap2]
+            ):
                 return False
         return True
 
@@ -207,14 +220,14 @@ class CAF[I]():
             raise ValueError(msg)
 
         (
-                self._front,
-                self._items[self._front],
-                self._cnt,
+            self._front,
+            self._items[self._front],
+            self._cnt,
         ) = (
-                (self._front - 1) % self._cap,
-                item,
-                self._cnt + 1,
-            )
+            (self._front - 1) % self._cap,
+            item,
+            self._cnt + 1,
+        )
 
     def pushr(self, item: I) -> None:
         """Push ``item`` on from Right.
@@ -227,14 +240,14 @@ class CAF[I]():
             raise ValueError(msg)
 
         (
-                self._rear,
-                self._items[self._rear],
-                self._cnt,
+            self._rear,
+            self._items[self._rear],
+            self._cnt,
         ) = (
-                (self._rear + 1) % self._cap,
-                item,
-                self._cnt + 1,
-            )
+            (self._rear + 1) % self._cap,
+            item,
+            self._cnt + 1,
+        )
 
     def popl(self) -> I:
         """Pop single item off from left side.
@@ -244,30 +257,30 @@ class CAF[I]():
         """
         if self._cnt > 1:
             (
-                    d,
-                    self._items[self._front],
-                    self._front,
-                    self._cnt,
+                d,
+                self._items[self._front],
+                self._front,
+                self._cnt,
             ) = (
-                    self._items[self._front],
-                    None,
-                    (self._front + 1) % self._cap,
-                    self._cnt - 1,
-                )
+                self._items[self._front],
+                nada,
+                (self._front + 1) % self._cap,
+                self._cnt - 1,
+            )
         elif self._cnt == 1:
             (
-                    d,
-                    self._items[self._front],
-                    self._cnt,
-                    self._front,
-                    self._rear,
+                d,
+                self._items[self._front],
+                self._cnt,
+                self._front,
+                self._rear,
             ) = (
-                    self._items[self._front],
-                    None,
-                    0,
-                    0,
-                    self._cap - 1,
-                )
+                self._items[self._front],
+                nada,
+                0,
+                0,
+                self._cap - 1,
+            )
         else:
             msg = 'Method popl called on an empty CAF'
             raise ValueError(msg)
@@ -281,30 +294,30 @@ class CAF[I]():
         """
         if self._cnt > 1:
             (
-                    d,
-                    self._items[self._rear],
-                    self._rear,
-                    self._cnt,
+                d,
+                self._items[self._rear],
+                self._rear,
+                self._cnt,
             ) = (
-                    self._items[self._rear],
-                    None,
-                    (self._rear - 1) % self._cap,
-                    self._cnt - 1,
-                )
+                self._items[self._rear],
+                nada,
+                (self._rear - 1) % self._cap,
+                self._cnt - 1,
+            )
         elif self._cnt == 1:
             (
-                    d,
-                    self._items[self._front],
-                    self._cnt,
-                    self._front,
-                    self._rear,
+                d,
+                self._items[self._front],
+                self._cnt,
+                self._front,
+                self._rear,
             ) = (
-                    self._items[self._front],
-                    None,
-                    0,
-                    0,
-                    self._cap - 1,
-                )
+                self._items[self._front],
+                nada,
+                0,
+                0,
+                self._cap - 1,
+            )
         else:
             msg = 'Method popr called on an empty CAF'
             raise ValueError(msg)
@@ -337,7 +350,8 @@ class CAF[I]():
     def poplt(self, maximum: int) -> tuple[I, ...]:
         """Pop multiple items from left side of circular array.
 
-        :param maximum: Maximum number of items to pop, may pop less if not enough items.
+        :param maximum: Maximum number of items to pop, may pop less
+                        if not enough items.
         :returns: Tuple of items in the order popped, left to right.
         """
         item_list: list[I] = []
@@ -355,7 +369,8 @@ class CAF[I]():
     def poprt(self, maximum: int) -> tuple[I, ...]:
         """Pop multiple items from right side of circular array.
 
-        :param maximum: Maximum number of items to pop, may pop less if not enough items.
+        :param maximum: Maximum number of items to pop, may pop less
+                        if not enough items.
         :returns: Tuple of items in the order popped, right to left.
         """
         item_list: list[I] = []
@@ -388,60 +403,74 @@ class CAF[I]():
         for _ in range(n, 0, -1):
             self.pushl(self.popr())
 
-    def map[U](self, f: Callable[[I], U]) -> "CAF[U]":
+    def map[U](self, f: Callable[[I], U]) -> 'CAF[U]':
         """Apply function ``f`` over the circular array's contents,
 
         :param f: Callable from type ``I`` to type ``U``.
         :returns: New fixed capacity circular array instance.
         """
-        return CAF(map(f, self), self._cap)
+        return CAF(map(f, self), cap = self._cap)
 
-    def foldl[L](self, f: Callable[[L, I], L], start: L | None = None) -> L:
+    @overload
+    def foldl[L](self, f: Callable[[I, I], I]) -> I: ...
+    @overload
+    def foldl[L](self, f: Callable[[L, I], L], start: L) -> L: ...
+
+    def foldl[L](self, f: Callable[[L, I], L], start: L | NoValue = nada) -> L:
         """Fold left with a function and optional stating item.
 
-        :param f: Folding function, first argument to ``f`` is for the accumulator.
+        :param f: Folding function, first argument to ``f`` is for
+                  the accumulator.
         :param start: Optional starting item.
         :returns: Reduced value produced by the left fold.
-        :raises ValueError: When circular array empty and no starting item given.
+        :raises ValueError: When circular array empty and no starting
+                            item given.
         """
         if self._cnt == 0:
-            if start is None:
+            if start is nada:
                 msg = 'Method foldl called on an empty CAF without a start item.'
                 raise ValueError(msg)
-            return start
+            return cast(L, start)
 
-        if start is None:
+        if start is nada:
             acc = cast(L, self[0])  # in this case D = L
             for idx in range(1, self._cnt):
                 acc = f(acc, self[idx])
             return acc
 
-        acc = start
+        acc = cast(L, start)
         for d in self:
             acc = f(acc, d)
         return acc
 
-    def foldr[R](self, f: Callable[[I, R], R], start: R | None = None) -> R:
+    @overload
+    def foldr[R](self, f: Callable[[I, I], I]) -> I: ...
+    @overload
+    def foldr[R](self, f: Callable[[I, R], R], start: R) -> R: ...
+
+    def foldr[R](self, f: Callable[[I, R], R], start: R | NoValue = nada) -> R:
         """Fold right with a function and an optional starting item.
 
-        :param f: Folding function, second argument to ``f`` is for the accumulator.
+        :param f: Folding function, second argument to ``f`` is for
+                  the accumulator.
         :param start: Optional starting item.
         :returns: Reduced value produced by the right fold.
-        :raises ValueError: When circular array empty and no starting item given.
+        :raises ValueError: When circular array empty and no starting
+                            item given.
         """
         if self._cnt == 0:
-            if start is None:
+            if start is nada:
                 msg = 'Method foldr called on empty CAF without initial value.'
                 raise ValueError(msg)
-            return start
+            return cast(R, start)
 
-        if start is None:
+        if start is nada:
             acc = cast(R, self[-1])  # in this case D = R
             for idx in range(self._cnt - 2, -1, -1):
                 acc = f(self[idx], acc)
             return acc
 
-        acc = start
+        acc = cast(R, start)
         for d in reversed(self):
             acc = f(d, acc)
         return acc
@@ -456,16 +485,16 @@ class CAF[I]():
     def empty(self) -> None:
         """Empty the circular array."""
         (
-                self._items,
-                self._front,
-                self._rear,
-                self._cnt,
+            self._items,
+            self._front,
+            self._rear,
+            self._cnt,
         ) = (
-                [None] * self._cap,
-                0,
-                self._cap - 1,
-                0,
-            )
+            [nada] * self._cap,
+            0,
+            self._cap - 1,
+            0,
+        )
 
     def fraction_filled(self) -> float:
         """Find fraction of the storage capacity which is filled.
@@ -475,11 +504,11 @@ class CAF[I]():
         return self._cnt / self._cap
 
 
-def caf[T](*items: T, capacity: int = 2) -> CAF[T]:
+def caf[T](*items: T, cap: int = 2) -> CAF[T]:
     """Produce a circular array from a variable number of arguments.
 
     :param items: Initial items for a new fixed capacity :circular array.
-    :param capacity: The minimum storage capacity to set.
+    :param cap: The minimum storage capacity to set.
     :returns: New fixed storage capacity circular array.
     """
-    return CAF(items, capacity = capacity)
+    return CAF(items, cap=cap)
